@@ -91,10 +91,8 @@ public:
      * that the instance can be deleted or set up to a new org_kde_plasma_shell interface
      * once there is a new connection available.
      *
-     * It is suggested to connect this method to ConnectionThread::connectionDied:
-     * @code
-     * connect(connection, &ConnectionThread::connectionDied, shell, &PlasmaShell::destroy);
-     * @endcode
+     * This method is automatically invoked when the Registry which created this
+     * PlasmaShell gets destroyed.
      *
      * Right before the data is destroyed, the signal interfaceAboutToBeDestroyed is emitted.
      *
@@ -176,6 +174,13 @@ private:
  *
  * To create an instance use PlasmaShell::createSurface.
  *
+ * A PlasmaShellSurface is a privileged Surface which can add further hints to the
+ * Wayland server about it's position and the usage role. The Wayland server is allowed
+ * to ignore all requests.
+ *
+ * Even if a PlasmaShellSurface is created for a Surface a normal ShellSurface (or similar)
+ * needs to be created to have the Surface mapped as a window by the Wayland server.
+ *
  * @see PlasmaShell
  * @see Surface
  **/
@@ -224,26 +229,54 @@ public:
      */
     static PlasmaShellSurface *get(Surface *surf);
 
+    /**
+     * @returns @c true if managing a org_kde_plasma_surface.
+     **/
     bool isValid() const;
     operator org_kde_plasma_surface*();
     operator org_kde_plasma_surface*() const;
 
+    /**
+     * Describes possible roles this PlasmaShellSurface can have.
+     * The role can be used by the Wayland server to e.g. change the stacking order accordingly.
+     **/
     enum class Role {
-        Normal,
-        Desktop,
-        Panel,
-        OnScreenDisplay
+        Normal, ///< A normal Surface
+        Desktop, ///< The Surface represents a desktop, normally stacked below all other surfaces
+        Panel, ///< The Surface represents a panel (dock), normally stacked above normal surfaces
+        OnScreenDisplay, ///< The Surface represents an on screen display, like a volume changed notification
+        Notification, ///< The Surface represents a notification @since 5.24
+        ToolTip ///< The Surface represents a tooltip @since 5.24
     };
+    /**
+     * Changes the requested Role to @p role.
+     * @see role
+     **/
     void setRole(Role role);
+    /**
+     * @returns The requested Role, default value is @c Role::Normal.
+     * @see setRole
+     **/
     Role role() const;
+    /**
+     * Requests to position this PlasmaShellSurface at @p point in global coordinates.
+     **/
     void setPosition(const QPoint &point);
 
+    /**
+     * Describes how a PlasmaShellSurface with role @c Role::Panel should behave.
+     * @see Role
+     **/
     enum class PanelBehavior {
         AlwaysVisible,
         AutoHide,
         WindowsCanCover,
         WindowsGoBelow
     };
+    /**
+     * Sets the PanelBehavior for a PlasmaShellSurface with Role @c Role::Panel
+     * @see setRole
+     **/
     void setPanelBehavior(PanelBehavior behavior);
 
     /**
@@ -254,6 +287,64 @@ public:
      */
     void setSkipTaskbar(bool skip);
 
+    /**
+     * Requests to hide a surface with Role Panel and PanelBahvior AutoHide.
+     *
+     * Once the compositor has hidden the panel the signal {@link autoHidePanelHidden} gets
+     * emitted. Once it is shown again the signal {@link autoHidePanelShown} gets emitted.
+     *
+     * To show the surface again from client side use {@link requestShowAutoHidingPanel}.
+     *
+     * @see autoHidePanelHidden
+     * @see autoHidePanelShown
+     * @see requestShowAutoHidingPanel
+     * @since 5.28
+     **/
+    void requestHideAutoHidingPanel();
+
+    /**
+     * Requests to show a surface with Role Panel and PanelBahvior AutoHide.
+     *
+     * This request allows the client to show a surface which it previously
+     * requested to be hidden with {@link requestHideAutoHidingPanel}.
+     *
+     * @see autoHidePanelHidden
+     * @see autoHidePanelShown
+     * @see requestHideAutoHidingPanel
+     * @since 5.28
+     **/
+    void requestShowAutoHidingPanel();
+
+    /**
+     * Set whether a PlasmaShellSurface with Role Panel should get focus or not.
+     *
+     * By default a Panel does not take focus. With this request the compositor
+     * can be instructed to also pass focus to a panel
+     *
+     * @param takesFocus Set to @c true if the Panel should gain focus.
+     * @since 5.28
+     **/
+    void setPanelTakesFocus(bool takesFocus);
+
+Q_SIGNALS:
+    /**
+     * Emitted when the compositor hided an auto hiding panel.
+     * @see requestHideAutoHidingPanel
+     * @see autoHidePanelShown
+     * @see requestShowAutoHidingPanel
+     * @since 5.28
+     **/
+    void autoHidePanelHidden();
+
+    /**
+     * Emitted when the compositor showed an auto hiding panel.
+     * @see requestHideAutoHidingPanel
+     * @see autoHidePanelHidden
+     * @see requestShowAutoHidingPanel
+     * @since 5.28
+     **/
+    void autoHidePanelShown();
+
 private:
     friend class PlasmaShell;
     class Private;
@@ -263,4 +354,7 @@ private:
 }
 }
 
+Q_DECLARE_METATYPE(KWayland::Client::PlasmaShellSurface::Role)
+Q_DECLARE_METATYPE(KWayland::Client::PlasmaShellSurface::PanelBehavior)
+
 #endif
