@@ -38,6 +38,8 @@ License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 #include <linux/input.h>
 #endif
 
+#include <functional>
+
 namespace KWayland
 {
 
@@ -207,6 +209,24 @@ QVector<T *> interfacesForSurface(SurfaceInterface *surface, const QVector<T*> &
     }
     return ret;
 }
+
+template <typename T>
+static
+bool forEachInterface(SurfaceInterface *surface, const QVector<T*> &interfaces, std::function<void (T*)> method)
+{
+    if (!surface) {
+        return false;
+    }
+    bool calledAtLeastOne = false;
+    for (auto it = interfaces.begin(); it != interfaces.end(); ++it) {
+        if ((*it)->client() == surface->client() && (*it)->resource()) {
+            method(*it);
+            calledAtLeastOne = true;
+        }
+    }
+    return calledAtLeastOne;
+}
+
 }
 
 QVector<PointerInterface *> SeatInterface::Private::pointersForSurface(SurfaceInterface *surface) const
@@ -330,6 +350,9 @@ void SeatInterface::Private::endDrag(quint32 serial)
 {
     auto target = drag.target;
     QObject::disconnect(drag.destroyConnection);
+    if (drag.source) {
+        drag.source->dragSource()->dropPerformed();
+    }
     if (target) {
         target->drop();
         target->updateDragTarget(nullptr, serial);
@@ -871,6 +894,128 @@ void SeatInterface::relativePointerMotion(const QSizeF &delta, const QSizeF &del
     }
 }
 
+void SeatInterface::startPointerSwipeGesture(quint32 fingerCount)
+{
+    Q_D();
+    if (!d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    d->globalPointer.gestureSurface = QPointer<SurfaceInterface>(d->globalPointer.focus.surface);
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial, fingerCount] (PointerInterface *p) {
+            p->d_func()->startSwipeGesture(serial, fingerCount);
+        }
+    );
+}
+
+void SeatInterface::updatePointerSwipeGesture(const QSizeF &delta)
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [delta] (PointerInterface *p) {
+            p->d_func()->updateSwipeGesture(delta);
+        }
+    );
+}
+
+void SeatInterface::endPointerSwipeGesture()
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial] (PointerInterface *p) {
+            p->d_func()->endSwipeGesture(serial);
+        }
+    );
+    d->globalPointer.gestureSurface.clear();
+}
+
+void SeatInterface::cancelPointerSwipeGesture()
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial] (PointerInterface *p) {
+            p->d_func()->cancelSwipeGesture(serial);
+        }
+    );
+    d->globalPointer.gestureSurface.clear();
+}
+
+void SeatInterface::startPointerPinchGesture(quint32 fingerCount)
+{
+    Q_D();
+    if (!d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    d->globalPointer.gestureSurface = QPointer<SurfaceInterface>(d->globalPointer.focus.surface);
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial, fingerCount] (PointerInterface *p) {
+            p->d_func()->startPinchGesture(serial, fingerCount);
+        }
+    );
+}
+
+void SeatInterface::updatePointerPinchGesture(const QSizeF &delta, qreal scale, qreal rotation)
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [delta, scale, rotation] (PointerInterface *p) {
+            p->d_func()->updatePinchGesture(delta, scale, rotation);
+        }
+    );
+}
+
+void SeatInterface::endPointerPinchGesture()
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial] (PointerInterface *p) {
+            p->d_func()->endPinchGesture(serial);
+        }
+    );
+    d->globalPointer.gestureSurface.clear();
+}
+
+void SeatInterface::cancelPointerPinchGesture()
+{
+    Q_D();
+    if (d->globalPointer.gestureSurface.isNull()) {
+        return;
+    }
+    const quint32 serial = d->display->nextSerial();
+    forEachInterface<PointerInterface>(d->globalPointer.gestureSurface.data(), d->pointers,
+        [serial] (PointerInterface *p) {
+            p->d_func()->cancelPinchGesture(serial);
+        }
+    );
+    d->globalPointer.gestureSurface.clear();
+}
+
 void SeatInterface::keyPressed(quint32 key)
 {
     Q_D();
@@ -1152,19 +1297,20 @@ qint32 SeatInterface::touchDown(const QPointF &globalPosition)
         d->touchInterface.focus.touch->down(id, serial, globalPosition - d->touchInterface.focus.offset);
     } else if (id == 0 && focusedTouchSurface()) {
 #if HAVE_LINUX_INPUT_H
-        auto p = d->pointersForSurface(focusedTouchSurface());
-        if (p.isEmpty()) {
-            return id;
-        }
         const QPointF pos = globalPosition - d->touchInterface.focus.offset;
-        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
-            wl_pointer_send_enter((*it)->resource(), serial,
-                            focusedTouchSurface()->resource(),
-                            wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
-            wl_pointer_send_motion((*it)->resource(), timestamp(),
-                                    wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+        const bool result = forEachInterface<PointerInterface>(focusedTouchSurface(), d->pointers,
+            [this, pos, serial] (PointerInterface *p) {
+                wl_pointer_send_enter(p->resource(), serial,
+                                focusedTouchSurface()->resource(),
+                                wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+                wl_pointer_send_motion(p->resource(), timestamp(),
+                                        wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
 
-            wl_pointer_send_button((*it)->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_PRESSED);
+                wl_pointer_send_button(p->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_PRESSED);
+            }
+        );
+        if (!result) {
+            return id;
         }
 #endif
     }
@@ -1179,16 +1325,13 @@ void SeatInterface::touchMove(qint32 id, const QPointF &globalPosition)
     if (d->touchInterface.focus.touch && d->touchInterface.focus.surface) {
         d->touchInterface.focus.touch->move(id, globalPosition - d->touchInterface.focus.offset);
     } else if (id == 0 && focusedTouchSurface()) {
-        auto p = d->pointersForSurface(focusedTouchSurface());
-        if (p.isEmpty()) {
-            return;
-        }
-
         const QPointF pos = globalPosition - d->touchInterface.focus.offset;
-        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
-            wl_pointer_send_motion((*it)->resource(), timestamp(),
-                                   wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
-        }
+        forEachInterface<PointerInterface>(focusedTouchSurface(), d->pointers,
+            [this, pos] (PointerInterface *p) {
+                wl_pointer_send_motion(p->resource(), timestamp(),
+                                       wl_fixed_from_double(pos.x()), wl_fixed_from_double(pos.y()));
+            }
+        );
     } 
 }
 
@@ -1201,14 +1344,11 @@ void SeatInterface::touchUp(qint32 id)
     } else if (id == 0 && focusedTouchSurface()) {
 #if HAVE_LINUX_INPUT_H
         const quint32 serial = display()->nextSerial();
-        auto p = d->pointersForSurface(focusedTouchSurface());
-        if (p.isEmpty()) {
-            return;
-        }
-
-        for (auto it = p.constBegin(), end = p.constEnd(); it != end; ++it) {
-            wl_pointer_send_button((*it)->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_RELEASED);
-        }
+        forEachInterface<PointerInterface>(focusedTouchSurface(), d->pointers,
+            [this, serial] (PointerInterface *p) {
+                wl_pointer_send_button(p->resource(), serial, timestamp(), BTN_LEFT, WL_POINTER_BUTTON_STATE_RELEASED);
+            }
+        );
 #endif
     }
     d->touchInterface.ids.removeAll(id);
@@ -1300,7 +1440,7 @@ void SeatInterface::setFocusedTextInputSurface(SurfaceInterface *surface)
     }
     d->textInput.focus.textInput = t;
     if (d->textInput.focus.surface) {
-        d->textInput.focus.destroyConnection = connect(surface, &QObject::destroyed, this,
+        d->textInput.focus.destroyConnection = connect(surface, &Resource::aboutToBeUnbound, this,
             [this] {
                 setFocusedTextInputSurface(nullptr);
             }
